@@ -1,7 +1,5 @@
 #!/usr/bin/perl -w
 
-# $Id: Cron.pm,v 1.16 2006/11/27 13:42:52 roland Exp $
-
 =head1 NAME
 
 Cron - cron-like scheduler for Perl subroutines
@@ -37,7 +35,7 @@ Cron - cron-like scheduler for Perl subroutines
 =head1 DESCRIPTION
 
 This module provides a simple but complete cron like scheduler.  I.e this
-modules can be used for periodically executing Perl subroutines.  The dates and
+module can be used for periodically executing Perl subroutines.  The dates and
 parameters for the subroutines to be called are specified with a format known
 as crontab entry (see L<"METHODS">, C<add_entry()> and L<crontab(5)>)
 
@@ -81,7 +79,7 @@ BEGIN {
 }
 
 
-$VERSION = "0.98";
+$VERSION = "0.99";
 
 our $DEBUG = 0;
 my %STARTEDCHILD = ();
@@ -753,7 +751,9 @@ sub run
                       if $log;
                     $self->_update_queue($index);
                     next;
-                }  
+                }
+                # At least a safety airbag
+                $sleep = 1;
             }
             else
             {
@@ -1130,12 +1130,21 @@ sub _execute
 sub _update_queue 
 { 
     my $self = shift;
-    
     my $index = shift;
-    
     my $entry = $self->get_entry($index);
     
     my $new_time = $self->get_next_execution_time($entry->{time});
+    # Check, whether next execution time is *smaller* than the current time.
+    # This can happen during DST backflip:
+    my $now = time;
+    if ($new_time < $now) {
+        dbg "Adjusting time calculation because of DST back flip (new_time - now = ",$new_time - $now,")";
+        # We are adding hours as long as our target time is in the future
+        while ($new_time < $now) {
+            $new_time += 3600;
+        }
+    }
+
     dbg "Updating Queue: ",scalar(localtime($new_time));
     $self->{queue} = [ sort { $a->[1] <=> $b->[1] } @{$self->{queue}},[$index,$new_time] ];
     #  dbg "Queue now: ",Dumper($self->{queue});
@@ -1276,7 +1285,6 @@ sub _calc_time
         {
             $dest_hour = ($dest_mday == $now_mday ? $dest_hour : 0);
         }
-    
         # Check for minute
         if ($expanded->[0]->[0] ne '*') 
         {
@@ -1308,7 +1316,6 @@ sub _calc_time
                 $dest_min = 0;
             } 
         }
-
         # Check for seconds
         if ($expanded->[5])
         {
@@ -1360,7 +1367,7 @@ sub _calc_time
         {
             # Invalid date i.e. (02/30/2008). Retry it with another, possibly
             # valid date            
-            my $t = parsedate($date); print scalar(localtime($t)),"\n";
+            my $t = parsedate($date); # print scalar(localtime($t)),"\n";
             ($dest_hour,$dest_mday,$dest_mon,$dest_year,$dest_wday) =
               (localtime(parsedate(" + 1 second",NOW=>$t)))  [2,3,4,5,6];
             $dest_mon++;
@@ -1632,16 +1639,37 @@ sub _verify_expanded_cron_entry {
 
 =back
 
-=head1 TODO
+=head1 DST ISSUES
 
-=over 
+Daylight saving occurs typically twice a year: In the first switch, one hour is
+skipped. Any job which which triggers in this skipped hour will be fired in the
+next hour. So, when the DST switch goes from 2:00 to 3:00 a job would is
+scheduled for 2:43, then it will be executed at 3:43.
 
-=item *
+For the reverse backwards switch later in the year, the behaviour is
+undefined. Two possible behaviours can occur: For jobs triggered in short
+intervals, where the next execution time would fire in the extra hour as well,
+the job could be executed again or skipped in this extra hour. Currently,
+running C<Schedule::Cron> in C<MET> would skip the extra job, in C<PST8PDT> it
+would execute a second time. The reason is the way how L<Time::ParseDate>
+calculates epoch times for dates given like C<02:50:00 2009/10/25>. Should it
+return the seconds since 1970 for this time happening 'first', or for this time
+in the extra hour ? As it turns out, L<Time::ParseDate> returns the epoch time
+of the first occurence for C<PST8PDT> and for C<MET> it returns the second
+occurence. Unfortunately, there is no way to specify I<which> entry
+L<Time::ParseDate> should pick (until now). Of course, after all, this is
+obviously not L<Time::ParseDate>'s fault, since a simple date specification
+within the DST backswitch period B<is> ambigious. However, it would be nice if
+the parsing behaviour of L<Time::ParseDate> would be consistent across time
+zones (a ticket has be raised for fixing this). Then L<Schedule::Cron>'s
+behaviour within a DST backward switch would be consistent as well.
 
-Provide a C<reload()> method for reexaming the crontab file 
-
-=back
+Since changing the internal algorithm which worked now for over ten years would
+be too risky and I don't see any simple solution for this right now, it is
+likely that this I<undefined> behaviour will exist for some time. Maybe some
+hero is coming along and will fix this, but this is probably not me ;-)
 
+Sorry for that.
 
 =head1 LICENSE
 
@@ -1652,7 +1680,7 @@ modify it under the same terms as Perl itself.
 
 =head1 AUTHOR
 
-                                                    ...roland
+... roland
 
 =cut
 
